/*{ radio_cc1120.c *****************************************************************

 Program Name: cc1120
 
 This source is published under the GNU LESSER GENERAL PUBLIC LICENSE (LGPL).
 See file LEGAL.txt in this software package for details.

 Authors: <Sascha Poggemann>, <Gregor Rebel>

 Description: This is an implementation for the Texas Instruments cc1120 Tranceiver
 
}*/
//{ Private Defines/ TypeDefs ****************************************************

//}  Private Defines
//{ Includes *********************************************************************

#include "radio_cc1120.h"

//} Includes
//{ Private Structures/ Enums ****************************************************

//} Private Structures/ Enums
//{ Global Variables *************************************************************

// required to be called from here
extern void ttc_radio_set_operating_mode(u8_t LogicalIndex, ttc_radio_mode_e Mode);

ttc_array_define(ttc_mutex_smart_t*, TransactionLocks, TTC_AMOUNT_SPIS);

//} Global Variables
//{ Private Function prototypes **************************************************

#define LockTransaction(SPI_Index, Owner)  while (ttc_mutex_lock_smart( A(TransactionLocks, SPI_Index-1), -1, ( void(*)() ) Owner) != tqe_OK)
#define UnlockTransaction(SPI_Index)         ttc_mutex_unlock_smart( A(TransactionLocks, SPI_Index-1) )
u8_t Amount_NonEmptyPackets = 0; //D

//} Private Function prototypes
//{ Implemented Functions ********************************************************

void radio_cc1120_get_defaults(u8_t RadioIndex, ttc_radio_generic_t* Radio_Generic) {
    Assert_Radio(Radio_Generic, ec_InvalidArgument);
    Radio_Generic->LogicalIndex = RadioIndex;
    radio_cc1120_get_config(Radio_Generic); // will create a driver configuration on first call

    // ToDo: SP - fill Radio_Generic with default values
    Radio_Generic->MaxChannelTx    = CC1120_MAX_Channels;
    Radio_Generic->MaxChannelRx    = CC1120_MAX_Channels;
    Radio_Generic->MaxLevelTx      = CC1120_MAX_TX_LEVEL;
    Radio_Generic->MaxLevelRx      = CC1120_MAX_RX_LEVEL;
    Radio_Generic->Max_Header      = RADIO_CC1120_HEADER_SIZE;  // 2
    Radio_Generic->Max_Footer      = RADIO_CC1120_FOOTER_SIZE;  // 0
    Radio_Generic->Max_Payload     = RADIO_CC1120_PAYLOAD_SIZE; // 60
    Radio_Generic->ChannelRx       = 1;
    Radio_Generic->ChannelTx       = 1;
    Radio_Generic->Size_QueueTx    = 5;
    Radio_Generic->Size_QueueRx    = 5;
    Radio_Generic->TargetAddress   = 0xff; // send to broadcast address
}
void radio_cc1120_get_features(u8_t RadioIndex, ttc_radio_generic_t* Radio_Generic) {
    radio_cc1120_get_config(Radio_Generic); // checks pointer too

    // fill in default values
    radio_cc1120_get_defaults(RadioIndex, Radio_Generic);

    // add max values
    Radio_Generic->Flags.Bit.SleepMode_IDLE = 1;
    Radio_Generic->Flags.Bit.SleepMode_OFF  = 1;
    Radio_Generic->ChannelTx                = CC1120_MAX_Channels;
    Radio_Generic->ChannelRx                = CC1120_MAX_Channels;
    Radio_Generic->LevelTx                  = CC1120_MAX_TX_LEVEL;
    Radio_Generic->LevelRx                  = CC1120_MAX_RX_LEVEL;
    Radio_Generic->Flags.Bit.LBT            = 1;
}
ttc_radio_errorcode_e radio_cc1120_init(ttc_radio_generic_t* Radio_Generic) {
    if (0) ttc_task_begin_criticalsection(radio_cc1120_init); // disable multitasking during initialization (easier to debug)
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too
    ttc_radio_errorcode_e Error = tre_OK;
    u8_t State = 0;
    u8_t Level = 0;

    if (! A(TransactionLocks, DriverCfg->SPI_Index-1) )
        A(TransactionLocks, DriverCfg->SPI_Index-1) = ttc_mutex_create_smart();

    LockTransaction(DriverCfg->SPI_Index, radio_cc1120_init);

    // no SPI communication allowed before this point!
    radio_cc1120_spi_init(DriverCfg);

    if (!Error) { Level++; Error = radio_cc1120_spi_get_chip_id(DriverCfg); }
    if (!Error) { Level++; Error = radio_cc1120_spi_get_chip_version(DriverCfg); }
    if (!Error) { Level++; Error = radio_cc1120_spi_reset(DriverCfg); }
    while ( (State = _radio_cc1120_get_state(Radio_Generic)) != rcm_IDLE)
        Radio_Generic->TaskYield();

    // Calibrate radio according to errataradio_cc1120_init_SmartRF
    radio_cc1120_manualCalibration(Radio_Generic);
    if (!Error) { Level++; Error = radio_cc1120_init_SmartRF(Radio_Generic); }

    // set RXOFF + TXOFF modes according to cc1120 datasheet p.81
    // if (!Error) { Level++; Error = radio_cc1120_spi_register_single_write(DriverCfg, rcr_CC112X_MCSM1, 0x3F); }

    switch (DriverCfg->SPI_Index) { // configure external interrupt line for incoming data
#ifdef TTC_RADIO1
    case 1: {

        // initialize interrupt for receiving data from radio
        ttc_interrupt_init(tit_GPIO_Rising,
                           ttc_gpio_create_index8(TTC_RADIO1_PIN_GDO2),
                           _radio_cc1120_isr_receive,
                           (void*) Radio_Generic,
                           NULL, NULL
                           );
        ttc_interrupt_enable(tit_GPIO_Rising,
                             ttc_gpio_create_index8(TTC_RADIO1_PIN_GDO2),
                             TRUE
                             );
        break;
    }
#endif
#ifdef TTC_RADIO2
    case 2: {

        // initialize interrupt for receiving data from radio
        ttc_interrupt_init(tit_GPIO_Rising,
                           ttc_gpio_create_index8(TTC_RADIO2_PIN_GDO2),
                           _radio_cc1120_isr_receive,
                           (void*) Radio_Generic,
                           NULL, NULL
                           );
        ttc_interrupt_enable(tit_GPIO_Rising,
                             ttc_gpio_create_index8(TTC_RADIO2_PIN_GDO2),
                             TRUE
                             );
        break;
    }
#endif
#ifdef TTC_RADIO3
    case 3: {

        // initialize interrupt for receiving data from radio
        ttc_interrupt_init(tit_GPIO_Rising,
                           ttc_gpio_create_index8(TTC_RADIO3_PIN_GDO2),
                           _radio_cc1120_isr_receive,
                           (void*) Radio_Generic,
                           NULL, NULL
                           );
        ttc_interrupt_enable(tit_GPIO_Rising,
                             ttc_gpio_create_index8(TTC_RADIO3_PIN_GDO2),
                             TRUE
                             );
        break;
    }
#endif
#ifdef TTC_RADIO4
    case 4: {

        // initialize interrupt for receiving data from radio
        ttc_interrupt_init(tit_GPIO_Rising,
                           ttc_gpio_create_index8(TTC_RADIO4_PIN_GDO2),
                           _radio_cc1120_isr_receive,
                           (void*) Radio_Generic,
                           NULL, NULL
                           );
        ttc_interrupt_enable(tit_GPIO_Rising,
                             ttc_gpio_create_index8(TTC_RADIO4_PIN_GDO2),
                             TRUE
                             );
        break;
    }
#endif
#ifdef TTC_RADIO5
    case 5: {

        // initialize interrupt for receiving data from radio
        ttc_interrupt_init(tit_GPIO_Rising,
                           ttc_gpio_create_index8(TTC_RADIO5_PIN_GDO2),
                           _radio_cc1120_isr_receive,
                           (void*) Radio_Generic,
                           NULL, NULL
                           );
        ttc_interrupt_enable(tit_GPIO_Rising,
                             ttc_gpio_create_index8(TTC_RADIO5_PIN_GDO2),
                             TRUE
                             );
        break;
    }
#endif
    default: Assert(0, ec_InvalidConfiguration); // indexed radio not activated
    }

    UnlockTransaction(DriverCfg->SPI_Index);

    // function calls below lock their own transaction
    ttc_radio_set_operating_mode(Radio_Generic->LogicalIndex, trm_Idle);
    if (!Error) { Level++; Error = radio_cc1120_set_power_tx(Radio_Generic, Radio_Generic->LevelTx); }
    ttc_radio_set_operating_mode(Radio_Generic->LogicalIndex, trm_Receive);
    Assert(Error != tre_TimeOut, ec_InvalidConfiguration); // communication error on SPI-bus (check variable Level)
    Assert(Error == tre_OK,      ec_UNKNOWN);              // unknown error (check variable Level)

    if (0) ttc_task_end_criticalsection();
    return tre_OK;
}

#define VCDAC_START_OFFSET 2
#define FS_VCO2_INDEX 0
#define FS_VCO4_INDEX 1
#define FS_CHP_INDEX 2

ttc_radio_errorcode_e radio_cc1120_manualCalibration(ttc_radio_generic_t* Radio_Generic) {
    // Calibrate radio according to TexasInstruments errata sheet
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too
    ttc_radio_errorcode_e Error = tre_OK;
    u8_t original_fs_cal2;
    u8_t calResults_for_vcdac_start_high[3];
    u8_t calResults_for_vcdac_start_mid[3];
    u8_t marcstate;
    u8_t writeByte;
    
    // 1) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
    writeByte = 0x00;
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO2, writeByte);

    // 2) Start with high VCDAC (original VCDAC_START + 2):
    if (!Error) original_fs_cal2 = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_FS_CAL2);
    writeByte = original_fs_cal2 + VCDAC_START_OFFSET;
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CAL2, writeByte);

    // 3) Calibrate and wait for calibration to be done (radio back in IDLE state)
    if (!Error) Error = radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SCAL);
    
    do
    {
        marcstate = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_MARCSTATE);
    } while (marcstate != 0x41);
    
    // 4) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with high VCDAC_START value
    if (!Error) calResults_for_vcdac_start_high[FS_VCO2_INDEX] = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_FS_VCO2);
    if (!Error) calResults_for_vcdac_start_high[FS_VCO4_INDEX] = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_FS_VCO4);
    if (!Error) calResults_for_vcdac_start_high[FS_CHP_INDEX]  = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_FS_CHP);
    
    // 5) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
    writeByte = 0x00;
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO2, writeByte);

    // 6) Continue with mid VCDAC (original VCDAC_START):
    writeByte = original_fs_cal2;
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CAL2, writeByte);

    // 7) Calibrate and wait for calibration to be done (radio back in IDLE state)
    if (!Error) Error = radio_cc1120_spi_strobe(DriverCfg,  rcs_CC112X_SCAL);

    do
    {
        if (!Error) marcstate = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_MARCSTATE);
    } while (marcstate != 0x41);
    
    // 8) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with mid VCDAC_START value
    if (!Error) calResults_for_vcdac_start_mid[FS_VCO2_INDEX] = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_FS_VCO2);
    if (!Error) calResults_for_vcdac_start_mid[FS_VCO4_INDEX] = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_FS_VCO4);
    if (!Error) calResults_for_vcdac_start_mid[FS_CHP_INDEX]  = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_FS_CHP);

    // 9) Write back highest FS_VCO2 and corresponding FS_VCO and FS_CHP result
    if (calResults_for_vcdac_start_high[FS_VCO2_INDEX] > calResults_for_vcdac_start_mid[FS_VCO2_INDEX])
    {
        writeByte = calResults_for_vcdac_start_high[FS_VCO2_INDEX];
        if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO2, writeByte);
        writeByte = calResults_for_vcdac_start_high[FS_VCO4_INDEX];
        if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO4, writeByte);
        writeByte = calResults_for_vcdac_start_high[FS_CHP_INDEX];
        if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CHP, writeByte);
    }
    else
    {
        writeByte = calResults_for_vcdac_start_mid[FS_VCO2_INDEX];
        if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO2, writeByte);
        writeByte = calResults_for_vcdac_start_mid[FS_VCO4_INDEX];
        if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO4, writeByte);
        writeByte = calResults_for_vcdac_start_mid[FS_CHP_INDEX];
        if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CHP, writeByte);
    }
    return Error;
}
ttc_radio_errorcode_e radio_cc1120_init_SmartRF(ttc_radio_generic_t* Radio_Generic) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too

    // apply radio configuration
    ttc_radio_errorcode_e Error = tre_OK;
    LockTransaction(DriverCfg->SPI_Index, radio_cc1120_init_SmartRF);
    // Rf settings for CC1120
    //
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_IOCFG3,          SMARTRF_SETTING_IOCFG3);          //GPIO3 IO Pin Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_IOCFG2,          SMARTRF_SETTING_IOCFG2);          //GPIO2 IO Pin Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_IOCFG1,          SMARTRF_SETTING_IOCFG1);          //GPIO1 IO Pin Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_IOCFG0,          SMARTRF_SETTING_IOCFG0);          //GPIO0 IO Pin Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SYNC3,           SMARTRF_SETTING_SYNC3);           //Sync Word Configuration [31:24]
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SYNC2,           SMARTRF_SETTING_SYNC2);           //Sync Word Configuration [23:16]
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SYNC1,           SMARTRF_SETTING_SYNC1);           //Sync Word Configuration [15:8]
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SYNC0,           SMARTRF_SETTING_SYNC0);           //Sync Word Configuration [7:0]
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SYNC_CFG1,       SMARTRF_SETTING_SYNC_CFG1);       //Sync Word Detection Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SYNC_CFG0,       SMARTRF_SETTING_SYNC_CFG0);       //Sync Word Length Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_DEVIATION_M,     SMARTRF_SETTING_DEVIATION_M);     //Frequency Deviation Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_MODCFG_DEV_E,    SMARTRF_SETTING_MODCFG_DEV_E);   //Modulation Format and Frequency Deviation Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_DCFILT_CFG,      SMARTRF_SETTING_DCFILT_CFG);      //Digital DC Removal Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PREAMBLE_CFG1,   SMARTRF_SETTING_PREAMBLE_CFG1);   //Preamble Length Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PREAMBLE_CFG0,   SMARTRF_SETTING_PREAMBLE_CFG0);   //
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_FREQ_IF_CFG,     SMARTRF_SETTING_FREQ_IF_CFG);     //RX Mixer Frequency Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_IQIC,            SMARTRF_SETTING_IQIC);            //Digital Image Channel Compensation Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_CHAN_BW,         SMARTRF_SETTING_CHAN_BW);         //Channel Filter Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_MDMCFG1,         SMARTRF_SETTING_MDMCFG1);         //General Modem Parameter Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_MDMCFG0,         SMARTRF_SETTING_MDMCFG0);         //General Modem Parameter Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_DRATE2,          SMARTRF_SETTING_DRATE2);          //Data Rate Configuration Exponent and Mantissa [19:16]
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_DRATE1,          SMARTRF_SETTING_DRATE1);          //Data Rate Configuration Mantissa [15:8]
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_DRATE0,          SMARTRF_SETTING_DRATE0);          //Data Rate Configuration Mantissa [7:0]
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_AGC_REF,         SMARTRF_SETTING_AGC_REF);         //AGC Reference Level Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_AGC_CS_THR,      SMARTRF_SETTING_AGC_CS_THR);      //Carrier Sense Threshold Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_AGC_GAIN_ADJUST, SMARTRF_SETTING_AGC_GAIN_ADJUST); //RSSI Offset Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_AGC_CFG3,        SMARTRF_SETTING_AGC_CFG3);        //AGC Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_AGC_CFG2,        SMARTRF_SETTING_AGC_CFG2);        //AGC Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_AGC_CFG1,        SMARTRF_SETTING_AGC_CFG1);        //AGC Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_AGC_CFG0,        SMARTRF_SETTING_AGC_CFG0);        //AGC Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_FIFO_CFG,        SMARTRF_SETTING_FIFO_CFG);        //FIFO Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_DEV_ADDR,        SMARTRF_SETTING_DEV_ADDR);        //Device Address Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SETTLING_CFG,    SMARTRF_SETTING_SETTLING_CFG);    //Frequency Synthesizer Calibration and Settling Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_FS_CFG,          SMARTRF_SETTING_FS_CFG);          //Frequency Synthesizer Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_WOR_CFG1,        SMARTRF_SETTING_WOR_CFG1);        //eWOR Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_WOR_CFG0,        SMARTRF_SETTING_WOR_CFG0);        //eWOR Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_WOR_EVENT0_MSB,  SMARTRF_SETTING_WOR_EVENT0_MSB);  //Event 0 Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_WOR_EVENT0_LSB,  SMARTRF_SETTING_WOR_EVENT0_LSB);  //Event 0 Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PKT_CFG2,        SMARTRF_SETTING_PKT_CFG2);        //Packet Configuration, Reg 2
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PKT_CFG1,        SMARTRF_SETTING_PKT_CFG1);        //Packet Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PKT_CFG0,        SMARTRF_SETTING_PKT_CFG0);        //Packet Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_RFEND_CFG1,      SMARTRF_SETTING_RFEND_CFG1);      //RFEND Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_RFEND_CFG0,      SMARTRF_SETTING_RFEND_CFG0);      //RFEND Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PA_CFG2,         SMARTRF_SETTING_PA_CFG2);         //Power Amplifier Configuration, Reg 2
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PA_CFG1,         SMARTRF_SETTING_PA_CFG1);         //Power Amplifier Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PA_CFG0,         SMARTRF_SETTING_PA_CFG0);         //Power Amplifier Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PKT_LEN,         SMARTRF_SETTING_PKT_LEN);         //Packet Length Configuration

    //extended configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IF_MIX_CFG,      SMARTRF_SETTING_IF_MIX_CFG);      //IF Mix Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FREQOFF_CFG,     SMARTRF_SETTING_FREQOFF_CFG);     //Frequency Offset Correction Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_TOC_CFG,         SMARTRF_SETTING_TOC_CFG);         //Timing Offset Correction Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_MARC_SPARE,      SMARTRF_SETTING_MARC_SPARE);      //MARC Spare
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ECG_CFG,         SMARTRF_SETTING_ECG_CFG);         //External Clock Frequency Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_SOFT_TX_DATA_CFG,SMARTRF_SETTING_SOFT_TX_DATA_CFG);//Soft TX Data Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_EXT_CTRL,        SMARTRF_SETTING_EXT_CTRL);        //External Control Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RCCAL_FINE,      SMARTRF_SETTING_RCCAL_FINE);      //RC Oscillator Calibration (fine)
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RCCAL_COARSE,    SMARTRF_SETTING_RCCAL_COARSE);    //RC Oscillator Calibration (coarse)
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RCCAL_OFFSET,    SMARTRF_SETTING_RCCAL_OFFSET);    //RC Oscillator Calibration Clock Offset
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FREQOFF1,        SMARTRF_SETTING_FREQOFF1);        //Frequency Offset (MSB)
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FREQOFF0,        SMARTRF_SETTING_FREQOFF0);        //Frequency Offset (LSB)
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FREQ2,           SMARTRF_SETTING_FREQ2);           //Frequency Configuration [23:16]
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FREQ1,           SMARTRF_SETTING_FREQ1);           //Frequency Configuration [15:8]
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FREQ0,           SMARTRF_SETTING_FREQ0);           //Frequency Configuration [7:0]
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IF_ADC2,         SMARTRF_SETTING_IF_ADC2);         //Analog to Digital Converter Configuration, Reg 2
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IF_ADC1,         SMARTRF_SETTING_IF_ADC1);         //Analog to Digital Converter Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IF_ADC0,         SMARTRF_SETTING_IF_ADC0);         //Analog to Digital Converter Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_DIG1,         SMARTRF_SETTING_FS_DIG1);         //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_DIG0,         SMARTRF_SETTING_FS_DIG0);         //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CAL3,         SMARTRF_SETTING_FS_CAL3);         //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CAL2,         SMARTRF_SETTING_FS_CAL2);         //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CAL1,         SMARTRF_SETTING_FS_CAL1);         //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CAL0,         SMARTRF_SETTING_FS_CAL0);         //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_CHP,          SMARTRF_SETTING_FS_CHP);          //Charge Pump Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_DIVTWO,       SMARTRF_SETTING_FS_DIVTWO);       //Divide by 2
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_DSM1,         SMARTRF_SETTING_FS_DSM1);         //Digital Synthesizer Module Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_DSM0,         SMARTRF_SETTING_FS_DSM0);         //Digital Synthesizer Module Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_DVC1,         SMARTRF_SETTING_FS_DVC1);         //Divider Chain Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_DVC0,         SMARTRF_SETTING_FS_DVC0);         //Divider Chain Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_LBI,          SMARTRF_SETTING_FS_LBI);          //Local Bias Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_PFD,          SMARTRF_SETTING_FS_PFD);          //Phase Frequency Detector Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_PRE,          SMARTRF_SETTING_FS_PRE);          //Prescaler Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_REG_DIV_CML,  SMARTRF_SETTING_FS_REG_DIV_CML);  //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_SPARE,        SMARTRF_SETTING_FS_SPARE);        //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO4,         SMARTRF_SETTING_FS_VCO4);         //VCO Configuration, Reg 4
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO3,         SMARTRF_SETTING_FS_VCO3);         //VCO Configuration, Reg 3
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO2,         SMARTRF_SETTING_FS_VCO2);         //VCO Configuration, Reg 2
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO1,         SMARTRF_SETTING_FS_VCO1);         //VCO Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FS_VCO0,         SMARTRF_SETTING_FS_VCO0);         //VCO Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_GBIAS6,          SMARTRF_SETTING_GBIAS6);          //Global Bias Configuration, Reg 6
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_GBIAS5,          SMARTRF_SETTING_GBIAS5);          //Global Bias Configuration, Reg 5
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_GBIAS4,          SMARTRF_SETTING_GBIAS4);          //Global Bias Configuration, Reg 4
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_GBIAS3,          SMARTRF_SETTING_GBIAS3);          //Global Bias Configuration, Reg 3
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_GBIAS2,          SMARTRF_SETTING_GBIAS2);          //Global Bias Configuration, Reg 2
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_GBIAS1,          SMARTRF_SETTING_GBIAS1);          //Global Bias Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_GBIAS0,          SMARTRF_SETTING_GBIAS0);          //Global Bias Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IFAMP,           SMARTRF_SETTING_IFAMP);           //Intermediate Frequency Amplifier Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_LNA,             SMARTRF_SETTING_LNA);             //Low Noise Amplifier Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RXMIX,           SMARTRF_SETTING_RXMIX);           //RX Mixer Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_XOSC5,           SMARTRF_SETTING_XOSC5);           //Crystal Oscillator Configuration, Reg 5
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_XOSC4,           SMARTRF_SETTING_XOSC4);           //Crystal Oscillator Configuration, Reg 4
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_XOSC3,           SMARTRF_SETTING_XOSC3);           //Crystal Oscillator Configuration, Reg 3
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_XOSC2,           SMARTRF_SETTING_XOSC2);           //Crystal Oscillator Configuration, Reg 2
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_XOSC1,           SMARTRF_SETTING_XOSC1);           //Crystal Oscillator Configuration, Reg 1
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_XOSC0,           SMARTRF_SETTING_XOSC0);           //Crystal Oscillator Configuration, Reg 0
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ANALOG_SPARE,    SMARTRF_SETTING_ANALOG_SPARE);    //
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_PA_CFG3,         SMARTRF_SETTING_PA_CFG3);         //Power Amplifier Configuration, Reg 3
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IRQ0M,           SMARTRF_SETTING_IRQ0M);           //IRQ0 Mask Configuration
    if (!Error) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IRQ0F,           SMARTRF_SETTING_IRQ0F);           //IRQ0 Flag

    // Status Registers
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_WOR_TIME1,       SMARTRF_SETTING_WOR_TIME1);       //eWOR Timer Status (MSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_WOR_TIME0,       SMARTRF_SETTING_WOR_TIME0);       //eWOR Timer Status (LSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_WOR_CAPTURE1,    SMARTRF_SETTING_WOR_CAPTURE1);    //eWOR Timer Capture (MSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_WOR_CAPTURE0,    SMARTRF_SETTING_WOR_CAPTURE0);    //eWOR Timer Capture (LSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_BIST,            SMARTRF_SETTING_BIST);            //MARC BIST
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_DCFILTOFFSET_I1, SMARTRF_SETTING_DCFILTOFFSET_I1); //DC Filter Offset I (MSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_DCFILTOFFSET_I0, SMARTRF_SETTING_DCFILTOFFSET_I0); //DC Filter Offset I (LSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_DCFILTOFFSET_Q1, SMARTRF_SETTING_DCFILTOFFSET_Q1); //DC Filter Offset Q (MSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_DCFILTOFFSET_Q0, SMARTRF_SETTING_DCFILTOFFSET_Q0); //DC Filter Offset Q (LSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IQIE_I1,         SMARTRF_SETTING_IQIE_I1);         //IQ Imbalance Value I (MSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IQIE_I0,         SMARTRF_SETTING_IQIE_I0);         //IQ Imbalance Value I (LSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IQIE_Q1,         SMARTRF_SETTING_IQIE_Q1);         //IQ Imbalance Value Q (MSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_IQIE_Q0,         SMARTRF_SETTING_IQIE_Q0);         //IQ Imbalance Value Q (LSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RSSI1,           SMARTRF_SETTING_RSSI1);           //Received Signal Strength Indicator (MSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RSSI0,           SMARTRF_SETTING_RSSI0);           //Received Signal Strength Indicator (LSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_MARCSTATE,       SMARTRF_SETTING_MARCSTATE);       //MARC State
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_LQI_VAL,         SMARTRF_SETTING_LQI_VAL);         //Link Quality Indicator Value
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_PQT_SYNC_ERR,    SMARTRF_SETTING_PQT_SYNC_ERR);    //Preamble and Sync Word Error
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_DEM_STATUS,      SMARTRF_SETTING_DEM_STATUS);      //Demodulator Status
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FREQOFF_EST1,    SMARTRF_SETTING_FREQOFF_EST1);    //Frequency Offset Estimate (MSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FREQOFF_EST0,    SMARTRF_SETTING_FREQOFF_EST0);    //Frequency Offset Estimate (LSB)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_AGC_GAIN3,       SMARTRF_SETTING_AGC_GAIN3);       //AGC Gain, Reg 3
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_AGC_GAIN2,       SMARTRF_SETTING_AGC_GAIN2);       //AGC Gain, Reg 2
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_AGC_GAIN1,       SMARTRF_SETTING_AGC_GAIN1);       //AGC Gain, Reg 1
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_AGC_GAIN0,       SMARTRF_SETTING_AGC_GAIN0);       //AGC Gain, Reg 0
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_SOFT_RX_DATA_OUT,SMARTRF_SETTING_SOFT_RX_DATA_OUT);//Soft Decision Symbol Data
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_SOFT_TX_DATA_IN, SMARTRF_SETTING_SOFT_TX_DATA_IN); //Soft TX Data Input Register
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ASK_SOFT_RX_DATA,SMARTRF_SETTING_ASK_SOFT_RX_DATA);//AGC ASK Soft Decision Output
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RNDGEN,          SMARTRF_SETTING_RNDGEN);          //Random Number Value
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_MAGN2,           SMARTRF_SETTING_MAGN2);           //Signal Magnitude after CORDIC [16]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_MAGN1,           SMARTRF_SETTING_MAGN1);           //Signal Magnitude after CORDIC [15:8]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_MAGN0,           SMARTRF_SETTING_MAGN0);           //Signal Magnitude after CORDIC [7:0]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ANG1,            SMARTRF_SETTING_ANG1);            //Signal Angular after CORDIC [9:8]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ANG0,            SMARTRF_SETTING_ANG0);            //Signal Angular after CORDIC [7:0]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_CHFILT_I2,       SMARTRF_SETTING_CHFILT_I2);       //Channel Filter Data Real Part [18:16]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_CHFILT_I1,       SMARTRF_SETTING_CHFILT_I1);       //Channel Filter Data Real Part [15:8]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_CHFILT_I0,       SMARTRF_SETTING_CHFILT_I0);       //Channel Filter Data Real Part [7:0]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_CHFILT_Q2,       SMARTRF_SETTING_CHFILT_Q2);       //Channel Filter Data Imaginary Part [18:16]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_CHFILT_Q1,       SMARTRF_SETTING_CHFILT_Q1);       //Channel Filter Data Imaginary Part [15:8]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_CHFILT_Q0,       SMARTRF_SETTING_CHFILT_Q0);       //Channel Filter Data Imaginary Part [7:0]
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_GPIO_STATUS,     SMARTRF_SETTING_GPIO_STATUS);     //GPIO Status
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FSCAL_CTRL,      SMARTRF_SETTING_FSCAL_CTRL);      //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_PHASE_ADJUST,    SMARTRF_SETTING_PHASE_ADJUST);    //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_PARTNUMBER,      SMARTRF_SETTING_PARTNUMBER);      //Part Number
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_PARTVERSION,     SMARTRF_SETTING_PARTVERSION);     //Part Revision
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_SERIAL_STATUS,   SMARTRF_SETTING_SERIAL_STATUS);   //Serial Status
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RX_STATUS,       SMARTRF_SETTING_RX_STATUS);       //RX Status
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_TX_STATUS,       SMARTRF_SETTING_TX_STATUS);       //TX Status
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_MARC_STATUS1,    SMARTRF_SETTING_MARC_STATUS1);    //MARC Status, Reg 1
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_MARC_STATUS0,    SMARTRF_SETTING_MARC_STATUS0);    //MARC Status, Reg 0
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_PA_IFAMP_TEST,   SMARTRF_SETTING_PA_IFAMP_TEST);   //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FSRF_TEST,       SMARTRF_SETTING_FSRF_TEST);       //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_PRE_TEST,        SMARTRF_SETTING_PRE_TEST);        //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_PRE_OVR,         SMARTRF_SETTING_PRE_OVR);         //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ADC_TEST,        SMARTRF_SETTING_ADC_TEST);        //ADC Test
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_DVC_TEST,        SMARTRF_SETTING_DVC_TEST);        //DVC Test
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ATEST,           SMARTRF_SETTING_ATEST);           //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ATEST_LVDS,      SMARTRF_SETTING_ATEST_LVDS);      //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_ATEST_MODE,      SMARTRF_SETTING_ATEST_MODE);      //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_XOSC_TEST1,      SMARTRF_SETTING_XOSC_TEST1);      //
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_XOSC_TEST0,      SMARTRF_SETTING_XOSC_TEST0);      //

    /* DATA FIFO Access Registers */
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RXFIRST,         SMARTRF_SETTING_RXFIRST);         //RX FIFO Pointer (first entry)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_TXFIRST,         SMARTRF_SETTING_TXFIRST);         //TX FIFO Pointer (first entry)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_RXLAST,          SMARTRF_SETTING_RXLAST);          //RX FIFO Pointer (last entry)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_TXLAST,          SMARTRF_SETTING_TXLAST);          //TX FIFO Pointer (last entry)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_NUM_TXBYTES,     SMARTRF_SETTING_NUM_TXBYTES);     //TX FIFO Status (occupied entries)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_NUM_RXBYTES,     SMARTRF_SETTING_NUM_RXBYTES);     //RX FIFO Status (occupied entries)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FIFO_NUM_TXBYTES,SMARTRF_SETTING_FIFO_NUM_TXBYTES);//TX FIFO Status (free entries)
    if (0) Error = radio_cc1120_spi_register_write_16(DriverCfg, rcr_CC112X_FIFO_NUM_RXBYTES,SMARTRF_SETTING_FIFO_NUM_RXBYTES);//RX FIFO Status (available bytes)

    //final test
    u8_t Compare = 0;
    if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_DEV_ADDR, &Compare, cam_single_read, 1); //  0x01; // Device address.
    Assert_Radio(Compare == SMARTRF_SETTING_DEV_ADDR, ec_DeviceNotFound); // written value could not be verified

    UnlockTransaction(DriverCfg->SPI_Index);
    return Error;
}
ttc_radio_errorcode_e radio_cc1120_packet_send(ttc_radio_generic_t* Radio_Generic, u8_t TargetAddress, u8_t Size, u8_t* Data) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too
    Assert_Radio(Size, ec_InvalidArgument);
    Assert_Radio(Data, ec_InvalidArgument);

    ttc_radio_errorcode_e Error = tre_OK;
    Base_t StartTime =0;
    u16_t TimeTxPreparation = 0 ;

    Assert_Radio(Size, ec_InvalidArgument);
    Assert_Radio(Size <= RADIO_CC1120_PAYLOAD_SIZE, ec_InvalidArgument);

    LockTransaction(DriverCfg->SPI_Index, radio_cc1120_packet_send);

    if(1){ //D //disable complete radio transmission
        if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SINGLE_TXFIFO, Size + 1);
        Assert_Radio(Error == tre_OK, ec_UNKNOWN);
        if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_SINGLE_TXFIFO, TargetAddress);
        Assert_Radio(Error == tre_OK, ec_UNKNOWN);
        if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_SINGLE_TXFIFO, Data, cam_burst_write, Size);
        Assert_Radio(Error == tre_OK, ec_UNKNOWN);

        //ttc_task_begin_criticalsection();

        if (!Error) Error = radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_STX);

        u8_t TransmitState = 0;
        StartTime = ttc_task_get_tick_count();
        while ( (TransmitState = _radio_cc1120_get_state(Radio_Generic)) != rcm_TX) { //wait for radio to reach TX State
            switch(TransmitState){
            case rcm_TXFIFO_UNDERFLOW:
                return _radio_cc1120_handle_error(Radio_Generic, tre_TxFIFO_Underrun, TRUE);
            case rcm_RXFIFO_OVERFLOW:
                return _radio_cc1120_handle_error(Radio_Generic, tre_RxFIFO_Overrun, TRUE);
            case rcm_IDLE:
                return _radio_cc1120_handle_error(Radio_Generic, tre_UnexpectedState, TRUE);
            case rcm_RX:
                return _radio_cc1120_handle_error(Radio_Generic, tre_UnexpectedState, TRUE);
            default :
                break;

            }

            if((ttc_task_get_tick_count() - StartTime)>RADIO_CC1120_TRANSMIT_TIMEOUT)
                return _radio_cc1120_handle_error(Radio_Generic, tre_TX_Timeout, TRUE);

            Radio_Generic->TaskYield();
        }
        TimeTxPreparation = ttc_task_get_tick_count() - StartTime;

        while ( (TransmitState = _radio_cc1120_get_state(Radio_Generic)) != rcm_RX) { //wait for radio to reach RX State ttc_task_get_tick_count() always returns the same value: 11130
            switch(TransmitState){
            case rcm_TXFIFO_UNDERFLOW:
                return _radio_cc1120_handle_error(Radio_Generic, tre_TxFIFO_Underrun, TRUE);
            case rcm_RXFIFO_OVERFLOW:
                return _radio_cc1120_handle_error(Radio_Generic, tre_RxFIFO_Overrun, TRUE);
            case rcm_IDLE:
                return _radio_cc1120_handle_error(Radio_Generic, tre_UnexpectedState, TRUE);
            default :
                break;
            }

            Radio_Generic->TaskYield();
        }
        //ttc_task_end_criticalsection();
#ifdef TTC_DEBUG_RADIO
        u16_t TimeTxMode = ttc_task_get_tick_count() - StartTime - TimeTxPreparation;

        if (Radio_Generic->Max_TimeTxMode < TimeTxMode)
            Radio_Generic->Max_TimeTxMode = TimeTxMode;
        if (Radio_Generic->Max_TimeTxPreparation < TimeTxPreparation)
            Radio_Generic->Max_TimeTxPreparation = TimeTxPreparation;
#endif
    } //disable complete radio transmission

    UnlockTransaction(DriverCfg->SPI_Index);

    return Error;
}
u8_t radio_cc1120_packet_receive(ttc_radio_generic_t* Radio_Generic, u8_t* Buffer, u8_t MaxSize, u8_t* TargetAdress, u8_t* RSSI) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too
    ttc_radio_errorcode_e Error = tre_OK;
    Assert_Radio(RADIO_CC1120_PAYLOAD_SIZE <= MaxSize, ec_InvalidArgument); // Buffer[] is too small for this packet

    LockTransaction(DriverCfg->SPI_Index, radio_cc1120_packet_receive);

    u8_t RX_BytesPending = 0 ;
    if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_NUM_RXBYTES, &RX_BytesPending, cam_single_read, 1);
    u8_t PacketSize = 0;

    if (RX_BytesPending) {
        RX_BytesPending = RX_BytesPending& 0x7F; //RX_BytesPending contains MSB that indicates if RXFIFO is overflowed
        if (RX_BytesPending > CC1120_SIZE_RX_FIFO) { // invalid size: reread value via SPI
            //get bytes again, if a value > CC1120_SIZE_RX_FIFO is read from the RX Bytes Register.
            if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_NUM_RXBYTES, &RX_BytesPending, cam_single_read, 1); //D
            RX_BytesPending = RX_BytesPending& 0x7F;
        }
        if ( (RX_BytesPending > CC1120_SIZE_RX_FIFO) ||
             (RX_BytesPending > TTC_NETWORK_MAX_PAYLOAD_SIZE)
             ) { // invalid packet size: flush rx-buffer
            _radio_cc1120_handle_error(Radio_Generic, tre_RxFIFO_Overrun, FALSE);
        }
        Amount_NonEmptyPackets++; //D
        if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_SINGLE_RXFIFO, &PacketSize,  cam_single_read, 1);
        if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_SINGLE_RXFIFO, TargetAdress, cam_single_read, 1);
        PacketSize--; //remove Target Address


        if (PacketSize > RADIO_CC1120_PAYLOAD_SIZE) { // invalid size: flush fifo
            PacketSize = 0;
            Error = tre_RxCorruptPacket;
        }

        u8_t Bytes2Read = PacketSize;
        while (Bytes2Read--) {
            if (!Error) *Buffer++ = radio_cc1120_spi_register_read(DriverCfg, rcr_CC112X_SINGLE_RXFIFO);
        }
    }

    if (Error != tre_OK) {
        PacketSize = 0;
        _radio_cc1120_handle_error(Radio_Generic, Error, TRUE); // unlocks transaction
    }
    else
        UnlockTransaction(DriverCfg->SPI_Index);

    return PacketSize;
}
ttc_radio_errorcode_e radio_cc1120_set_device_address(ttc_radio_generic_t* Radio_Generic, u8_t NewDeviceAddress) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too

    ttc_radio_errorcode_e Error = tre_OK;
    if (Radio_Generic->DeviceAddress != NewDeviceAddress)
    {
        LockTransaction(DriverCfg->SPI_Index, radio_cc1120_packet_receive);
        if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_DEV_ADDR, &NewDeviceAddress, cam_single_write, 1 );
        Radio_Generic->DeviceAddress = NewDeviceAddress;
        UnlockTransaction(DriverCfg->SPI_Index);
    }
    return Error;
}
ttc_radio_errorcode_e radio_cc1120_set_channel(ttc_radio_generic_t* Radio_Generic, u32_t NewChannelIndex) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too
    ttc_radio_errorcode_e Error = tre_OK;

    LockTransaction(DriverCfg->SPI_Index, radio_cc1120_set_channel);
    //ToDo: Channel hopping without recalibration not supported by CC1120 refer to UserManual Page 67 for further Information
    UnlockTransaction(DriverCfg->SPI_Index);

    Assert_Radio(FALSE,ec_Debug); //D
    return Error;
}
ttc_radio_errorcode_e radio_cc1120_set_power_tx(ttc_radio_generic_t* Radio_Generic, s8_t Level) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too

    ttc_radio_errorcode_e Error = tre_OK;
    // determine power-setting for 868MHz according to cc1120- user manual Page 41
    u8_t PA_Power_Ramp_new = (Level+18)*2-1;
    u8_t PA_Power_Ramp_current = 0 ;

    LockTransaction(DriverCfg->SPI_Index, radio_cc1120_set_power_tx);

    if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_PA_CFG2, &PA_Power_Ramp_current, cam_single_read,1);//read current configuration
    PA_Power_Ramp_current &= ~0x3F; //reset the last 6 Bits (PA_CFG2.PA_POWER_RAMP)
    PA_Power_Ramp_current |= (PA_Power_Ramp_new& 0x3F);
    if (!Error) Error = radio_cc1120_spi_register_access(DriverCfg, rcr_CC112X_PA_CFG2, &PA_Power_Ramp_current, cam_single_write,1);//write back to register
    Radio_Generic->LevelTx = Level;

    UnlockTransaction(DriverCfg->SPI_Index);

    return Error;
}
ttc_radio_errorcode_e radio_cc1120_set_operating_mode(ttc_radio_generic_t* Radio_Generic) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too
    ttc_radio_errorcode_e Error = tre_OK;

    LockTransaction(DriverCfg->SPI_Index, radio_cc1120_set_operating_mode);
    radio_cc1120_marcstate_e CurrentMode = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_MARCSTATE);

    if (CurrentMode == rcm_RXFIFO_OVERFLOW) {
        UnlockTransaction(DriverCfg->SPI_Index); // _radio_cc1120_handle_error() might call us recursively!
        _radio_cc1120_handle_error(Radio_Generic, tre_RxFIFO_Overrun, FALSE);
        LockTransaction(DriverCfg->SPI_Index, radio_cc1120_set_operating_mode);
    }
    if (CurrentMode == rcm_TXFIFO_UNDERFLOW) {
        UnlockTransaction(DriverCfg->SPI_Index); // _radio_cc1120_handle_error() might call us recursively!
        _radio_cc1120_handle_error(Radio_Generic, tre_TxFIFO_Underrun, FALSE);
        LockTransaction(DriverCfg->SPI_Index, radio_cc1120_set_operating_mode);
    }

    switch (Radio_Generic->Mode) {
    case trm_Idle: {
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SIDLE);
        break;
    }
    case trm_Reset: {
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SRES);
        break;
    }
    case trm_PowerOff:
    case trm_Sleep: {
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SPWD);
        break;
    }
    case trm_PowerOn: {
        if (!Error) Error = radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SNOP);
        if (!Error) Error = radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SNOP);
        if (!Error) Error = radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SNOP);

        // reload configuration
        if (!Error) Error = radio_cc1120_init(Radio_Generic);
    }
    case trm_Transmit: {
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_STX);
        break;
    }
    case trm_Receive: {
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SRX);
        break;
    }
    case trm_TransmitReceive: {
    default:
            Assert_Radio(0, ec_InvalidArgument); // not supported by this radio
            break;
        }
    }

    UnlockTransaction(DriverCfg->SPI_Index);
    Base_t TimeOut = 10000;
    while (radio_cc1120_check_mode(Radio_Generic) && (TimeOut > 0)) {
        TimeOut--;
        ttc_task_yield();
    }
    if (TimeOut == 0)
        Error = tre_TimeOut;

    return Error;
}
BOOL radio_cc1120_check_mode(ttc_radio_generic_t* Radio_Generic) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too

    LockTransaction(DriverCfg->SPI_Index, radio_cc1120_set_operating_mode);
    BOOL IllegalMode = FALSE;

    switch (Radio_Generic->Mode) {
    case trm_Idle: {
        if (radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_MARCSTATE) != rcm_IDLE)
            IllegalMode = TRUE;
        break;
    }
    case trm_PowerOn: {
        radio_cc1120_marcstate_e CurrentMode = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_MARCSTATE);
        if ( (CurrentMode != rcm_IDLE) && (CurrentMode != rcm_TX) && (CurrentMode != rcm_RX) )
            IllegalMode = TRUE;
    }
    case trm_Transmit: {
        radio_cc1120_marcstate_e CurrentMode = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_MARCSTATE);
        if (CurrentMode != rcm_TX)
            IllegalMode = TRUE;
        break;
    }
    case trm_Receive: {
        radio_cc1120_marcstate_e CurrentMode = radio_cc1120_spi_register_read_extended(DriverCfg, rcr_CC112X_MARCSTATE);
        if (CurrentMode != rcm_RX)
            IllegalMode = TRUE;
        break;
    }
    default: break;
    }
    UnlockTransaction(DriverCfg->SPI_Index);

    return IllegalMode;
}
u8_t radio_cc1120_get_spi_index(u8_t RadioIndex) {
    Assert_Radio(RadioIndex > 0, ec_InvalidArgument); // logical index starts at 1

    switch (RadioIndex) {
#ifdef TTC_RADIO1_SPI_INDEX
    case 1: return TTC_RADIO1_SPI_INDEX;
#endif
#ifdef TTC_RADIO2_SPI_INDEX
    case 2: return TTC_RADIO2_SPI_INDEX;
#endif
#ifdef TTC_RADIO3_SPI_INDEX
    case 3: return TTC_RADIO3_SPI_INDEX;
#endif
#ifdef TTC_RADIO4_SPI_INDEX
    case 4: return TTC_RADIO4_SPI_INDEX;
#endif
#ifdef TTC_RADIO5_SPI_INDEX
    case 5: return TTC_RADIO5_SPI_INDEX;
#endif
    default: break;
    }
    Assert_Radio(0, ec_InvalidConfiguration); // indexed radio is not activated!
    return 0;
}
u8_t ISR_Call_DEBUG_VALUE = 0; //D
radio_cc1120_driver_t* radio_cc1120_get_config(ttc_radio_generic_t* Radio_Generic) {
    Assert_Radio(Radio_Generic, ec_InvalidArgument);

    radio_cc1120_driver_t* DriverCfg = (radio_cc1120_driver_t*) Radio_Generic->DriverCfg;
    if (!DriverCfg) {
        DriverCfg = (radio_cc1120_driver_t*) ttc_memory_alloc_zeroed( sizeof(radio_cc1120_driver_t) );
        Radio_Generic->DriverCfg = (ttc_radio_driver_t*) DriverCfg;
        DriverCfg->SPI_Index = radio_cc1120_get_spi_index(Radio_Generic->LogicalIndex);

        // ToDo: SP - fill out default driver configuration
        DriverCfg->RadioIndex = Radio_Generic->LogicalIndex;
        DriverCfg->Lock = ttc_mutex_create();
    }
    return DriverCfg;
}
ttc_radio_errorcode_e radio_cc1120_LBT_enable(ttc_radio_generic_t* Radio_Generic, bool enable){
    Assert_Radio(Radio_Generic, ec_InvalidArgument);

    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too
    ttc_radio_errorcode_e Error = tre_OK;
    u8_t currentPKT_CFG2 = 0 ;

    if(Radio_Generic->Flags.Bit.LBT != 0){
        LockTransaction(DriverCfg->SPI_Index, radio_cc1120_LBT_enable);
        if(enable == TRUE){ //LBT is supported
            //ToDo PKT_CFG2.CCA_MODE = 100b to enable LBT
            //UserManual Page 40
            currentPKT_CFG2 = radio_cc1120_spi_register_read(DriverCfg, rcr_CC112X_PKT_CFG2);
            currentPKT_CFG2 &= ~0b00011100; //clear CCA_Mode Bits
            currentPKT_CFG2 |= 0b10000; //set CCA_Mode = 0b100
            //ToDo: 0bxxxx generates GCC Warning maybe implement Registers so that single Bits can be set or cleared?!
            if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PKT_CFG2, currentPKT_CFG2);
        }else {
            u8_t PKT_CFG2origConfig = SMARTRF_SETTING_PKT_CFG2; //get original config from SmartRF settings
            PKT_CFG2origConfig &= 0b00011100; //clear other bits except CCA_Mode bits
            currentPKT_CFG2 = radio_cc1120_spi_register_read(DriverCfg, rcr_CC112X_PKT_CFG2);
            currentPKT_CFG2 &= ~0b00011100; //clear CCA_Mode Bits
            currentPKT_CFG2 |= PKT_CFG2origConfig; // apply original configuration form SmartRF settings
            //restore PKT_CFG2.CCA_Mode configuration from SmartRF config
            if (!Error) Error = radio_cc1120_spi_register_write_8(DriverCfg, rcr_CC112X_PKT_CFG2, currentPKT_CFG2);
        }
        UnlockTransaction(DriverCfg->SPI_Index);
    }else{
        return tre_NotImplemented;
    }

    return Error;
}
void _radio_cc1120_isr_receive(physical_index_t PhysicalIndex, void* Argument) {
    (void) PhysicalIndex;
    Assert_Radio(Argument, ec_InvalidArgument);
#ifdef TTC_DEBUG_RADIO
    //ttc_gpio_set(TTC_DEBUG_RADIO_PIN2);
    ISR_Call_DEBUG_VALUE = 1 - ISR_Call_DEBUG_VALUE; //D
    //? ttc_gpio_setn(TTC_DEBUG_RADIO_PIN2,ISR_Call_DEBUG_VALUE);//D
#endif
    ttc_radio_generic_t* Radio_Generic = (ttc_radio_generic_t*) Argument;
    if (Radio_Generic->MutexPacketReceived)
        ttc_mutex_unlock(Radio_Generic->MutexPacketReceived);

#ifdef TTC_DEBUG_RADIO
    //ttc_gpio_clr(TTC_DEBUG_RADIO_PIN2);
#endif
}
u8_t _radio_cc1120_get_state(ttc_radio_generic_t* Radio_Generic) {
    Assert_Radio(Radio_Generic, ec_InvalidArgument);
    radio_cc1120_driver_t* DriverCfg = (radio_cc1120_driver_t*) Radio_Generic->DriverCfg;

    u8_t MarcState;
    ttc_radio_errorcode_e Error = radio_cc1120_spi_register_access(DriverCfg,
                                                                   rcr_CC112X_MARCSTATE,
                                                                   &MarcState,
                                                                   cam_single_read,
                                                                   1
                                                                   );
    Assert_Radio(Error == tre_OK, ec_UNKNOWN);
    return MarcState;
}
ttc_radio_errorcode_e _radio_cc1120_handle_error(ttc_radio_generic_t* Radio_Generic, ttc_radio_errorcode_e Error, BOOL UnlockTransaction) {
    radio_cc1120_driver_t* DriverCfg = radio_cc1120_get_config(Radio_Generic); // checks pointer too

    ttc_radio_errorcode_e Error_return = tre_OK;

#ifdef TTC_DEBUG_RADIO
    Radio_Generic->Amount_Error_Handler_Calls++; //D
    Assert_Radio(Radio_Generic->Amount_Error_Handler_Calls<50, ec_Debug);//D
#endif
    if (Error == tre_TX_Timeout){
#ifdef TTC_DEBUG_RADIO
        Radio_Generic->Amount_Error_TX_Timeout++; //D
        Assert_Radio(Radio_Generic->Amount_Error_TX_Timeout<50, ec_Debug);//D
#endif
        if(_radio_cc1120_get_state(Radio_Generic) == rcm_RX){ // assume that we missed the TX State because radio is allready back in RX State
#ifdef TTC_DEBUG_RADIO
            Radio_Generic->Amount_Error_Missed_TXStates++; //D
#endif
            Error_return = tre_OK;
        }
        else {
            Assert_Radio(FALSE, ec_UNKNOWN);
            //assert if state is != rcm_RX because then we have an unexpected state
            //this assert could be replaced by another  _radio_cc1120_handle_error call with tre_UnexpectedState
            //this requires recursive locks.
        }
    } else if(Error == tre_RxCorruptPacket){
#ifdef TTC_DEBUG_RADIO
        Radio_Generic->Amount_Error_RxCorruptPacket++; //D
#endif
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SFRX);
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SRX);
    }
    else if(Error == tre_UnexpectedState){
#ifdef TTC_DEBUG_RADIO
        Radio_Generic->Amount_Error_UnexpectedState++; //D
#endif
        u8_t CurrentState = _radio_cc1120_get_state(Radio_Generic) ;
        if(CurrentState == rcm_TXFIFO_UNDERFLOW){
#ifdef TTC_DEBUG_RADIO
            Radio_Generic->Amount_Error_TXFIFO_UNDERFLOW++; //D
#endif
            radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SFTX);
        }
        else if(CurrentState == rcm_RXFIFO_OVERFLOW){
#ifdef TTC_DEBUG_RADIO
            Radio_Generic->Amount_Error_RXFIFO_OVERFLOW++; //D
#endif
            radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SFRX);
        }
        //ToDo: needs to bring back the radio to an known state.
    }
    else if(Error==tre_RxFIFO_Overrun){
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SFRX);
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SIDLE);
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SRX);
#ifdef TTC_DEBUG_RADIO
        Radio_Generic->Amount_Error_RXFIFO_OVERFLOW++; //D
#endif
    }
    else {
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SFTX);
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SFRX);
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SIDLE);
        if (!Error_return) Error_return = radio_cc1120_init(Radio_Generic);
        radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SRX);
    }
    if (Error_return)
        Assert_Halt_EC(ec_UNKNOWN); // Break here


    UnlockTransaction(DriverCfg->SPI_Index);

    ttc_radio_set_operating_mode(Radio_Generic->LogicalIndex, Radio_Generic->Mode); //switch radio back to its configured Mode

    if (UnlockTransaction){
        UnlockTransaction(DriverCfg->SPI_Index);
    }
    else {
        LockTransaction(DriverCfg->SPI_Index,_radio_cc1120_handle_error);
    }

    //    radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SFRX);
    //    radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SIDLE);
    //    radio_cc1120_spi_strobe(DriverCfg, rcs_CC112X_SRX);
    return Error_return;
}

//} Implemented Functions
