/*{ stm32_gpio.c ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Low Level Driver for General Purpose Input Output (GPIO) on STM32 architecture.
 * 
}*/

#include "stm32_gpio.h"

 GPIOMode_TypeDef stm32_map_Type(ttc_gpio_mode_e Type) {
    switch (Type) { // find best matching type for stm32-architecture
      case tgm_analog_in:                     return GPIO_Mode_AIN;
      case tgm_input_floating:                return GPIO_Mode_IN_FLOATING;
      case tgm_input_pull_down:               return GPIO_Mode_IPD;
      case tgm_input_pull_up:                 return GPIO_Mode_IPU;
      case tgm_output_open_drain:             return GPIO_Mode_Out_OD;
      case tgm_output_push_pull:              return GPIO_Mode_Out_PP;
      case tgm_alternate_function_push_pull:  return GPIO_Mode_AF_PP;
      case tgm_alternate_function_open_drain: return GPIO_Mode_AF_OD;
        
      default: Assert(0, ec_InvalidArgument);
    }
    
    return 0;
}
GPIOSpeed_TypeDef stm32_map_Speed(ttc_gpio_speed_e Speed) {
    switch (Speed) { // find best matching speed for stm32-architecture
      case tgs_Min:
      case tgs_2MHz:  return GPIO_Speed_2MHz;
      case tgs_10MHz: return GPIO_Speed_10MHz;
      case tgs_50MHz:
      case tgs_Max:   return GPIO_Speed_50MHz;
      default: Assert(0, ec_InvalidArgument);
    }
    
    return 0;
}
inline volatile u32_t* stm32_calc_peripheral_BitBandAddress(void* PeripheralWord, u8_t BitNumber) {
    return (u32_t*) (0x42000000 + 32 * (((u32_t) PeripheralWord)  - 0x40000000) + BitNumber * 4);
}

// direct port manipulations ---------------------------------------------
void stm32_gpio_init(GPIO_TypeDef* GPIOx, u8_t Pin, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed) {

    switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
      case (u32_t) GPIOA: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE); break;
      case (u32_t) GPIOB: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE); break;
      case (u32_t) GPIOC: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE); break;
      case (u32_t) GPIOD: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE); break;
      case (u32_t) GPIOE: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE, ENABLE); break;
      case (u32_t) GPIOF: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF, ENABLE); break;
#ifdef GPIOG
      case (u32_t) GPIOG: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOG, ENABLE); break;
#endif
      default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
    }

    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Mode =Type;
    GPIO_InitStructure.GPIO_Pin  =1 << Pin;
    GPIO_InitStructure.GPIO_Speed=Speed;
    GPIO_Init(GPIOx, &GPIO_InitStructure);

    switch (Type) {
      case GPIO_Mode_IPU: {
        break;
      }
      case GPIO_Mode_Out_PP: {
        //? GPIO_WriteBit(GPIOx, 1 << Pin, Bit_SET);
        break;
      }
      case GPIO_Mode_Out_OD: {
        stm32_gpio_set(GPIOx, Pin); // safe value
        break;
      }
      default: break;
    }
}
inline bool stm32_gpio_get(GPIO_TypeDef* GPIOx, u8_t Pin) {

    u32_t Data = GPIOx->IDR;
    if ( Data & (1 << Pin) ) return TRUE;
    return FALSE;
}
inline void stm32_gpio_clr(GPIO_TypeDef* GPIOx, u8_t Pin) {
    GPIOx->BRR |= 1 << Pin;
}
inline void stm32_gpio_set(GPIO_TypeDef* GPIOx, u8_t Pin) {
    GPIOx->BSRR |= 1 << Pin;
}

// port manipulations by variable ----------------------------------------
inline void stm32_gpio_variable(stm32_Port_t* Port, GPIO_TypeDef* GPIOx, u8_t Pin) {
    Port->GPIOx= GPIOx;
    Port->Pin  = Pin;
    Port->BRR  = stm32_calc_peripheral_BitBandAddress( (void*) &(GPIOx->BRR),  Pin);
    Port->BSRR = stm32_calc_peripheral_BitBandAddress( (void*) &(GPIOx->BSRR), Pin);
    Port->IDR  = stm32_calc_peripheral_BitBandAddress( (void*) &(GPIOx->IDR),  Pin);
}
void stm32_gpio_init_variable(stm32_Port_t* Port, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed) {
// #warning stm32_gpio_init_variable() is deprecated and will be replaced by stm32_gpio_init() soon!
    stm32_gpio_init(Port->GPIOx, Port->Pin, Type, Speed);
}
inline bool stm32_gpio_get_variable(stm32_Port_t* Port) {
    if (1)
        return *(Port->IDR); // untested
    else {
        u32_t Data = Port->GPIOx->IDR;
        if ( Data & (1 << Port->Pin) ) return TRUE;
        return FALSE;
    }
}
inline void stm32_gpio_set_variable(stm32_Port_t* Port) {
    *(Port->BSRR) = 1;
}
inline void stm32_gpio_clr_variable(stm32_Port_t* Port) {
    *(Port->BRR) = 1;
}

// convert GPIO+Pin <-> PhysicalIndex
u8_t stm32_gpio_create_index8(ttc_GPIO_BANK GPIOx, u8_t Pin) {

    switch ( (u32_t) GPIOx ) {

#ifdef GPIOA
    case (u32_t) GPIOA: return 0x10 | (0x0f & Pin);
#endif
#ifdef GPIOB
    case (u32_t) GPIOB: return 0x20 | (0x0f & Pin);
#endif
#ifdef GPIOC
    case (u32_t) GPIOC: return 0x30 | (0x0f & Pin);
#endif
#ifdef GPIOD
    case (u32_t) GPIOD: return 0x40 | (0x0f & Pin);
#endif
#ifdef GPIOE
    case (u32_t) GPIOE: return 0x50 | (0x0f & Pin);
#endif
#ifdef GPIOF
    case (u32_t) GPIOF: return 0x60 | (0x0f & Pin);
#endif
#ifdef GPIOG
    case (u32_t) GPIOG: return 0x70 | (0x0f & Pin);
#endif

    default: break;
    }

    return 0;
}
void stm32_gpio_from_index8(u8_t PhysicalIndex, GPIO_TypeDef** GPIOx, u8_t* Pin) {

    switch (PhysicalIndex & 0xf0) {

#ifdef GPIOA
    case 0x10: *GPIOx = GPIOA; break;
#endif
#ifdef GPIOB
    case 0x20: *GPIOx = GPIOB; break;
#endif
#ifdef GPIOC
    case 0x30: *GPIOx = GPIOC; break;
#endif
#ifdef GPIOD
    case 0x40: *GPIOx = GPIOD; break;
#endif
#ifdef GPIOE
    case 0x50: *GPIOx = GPIOE; break;
#endif
#ifdef GPIOF
    case 0x60: *GPIOx = GPIOF; break;
#endif
#ifdef GPIOG
    case 0x70: *GPIOx = GPIOG; break;
#endif

    default: *GPIOx = NULL;
    }

    *Pin = PhysicalIndex & 0x0f;
}
