#ifndef TTC_GPIO_H
#define TTC_GPIO_H

/*{ ttc_gpio.h ***********************************************
 
 Written by Gregor Rebel 2010-2012

 Architecture independent support for General Purpose Input Output (GPIO).

 Currently implemented architectures: stm32


 Example usage:
 
 Makefile
    COMPILE_OPTS += -DTTC_LED1=PIN_PA8
 
 Source
    #include "ttc_gpio.h"
 
    // use of constant gpios:
    ttc_gpio_init(TTC_LED1, tgm_output_push_pull);
    while (1) {
      ttc_gpio_set(TTC_LED1);
      ttc_gpio_clr(TTC_LED1);
    }

    // use of variable based gpios:
    ttc_Port_t LED;
    ttc_gpio_init(&LED, TTC_LED1, tgm_output_push_pull);
    while (1) {
      ttc_gpio_set_variable(&Port);
      ttc_gpio_clr_variable(&Port);
    }

    // use of shortcuts:
    while (1) {
      gset(TTC_LED1);
      gclr(TTC_LED1);
    }
    while (1) {
      gset_v(&Port);
      gclr_v(&Port);
    }
}*/
//{ includes

#include "ttc_basic.h"
#include "ttc_gpio_types.h"

//}includes
//{ Prototypes

/** initializes single port bit for use as input/ output with given speed by use of a state variable
 *
 * @param Port   points to structure to be filled with given data
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 * @param Type   mode to use (-> ttc_gpio_types.h:ttc_gpio_mode_e)
 * @param Speed  speed to use (-> ttc_gpio_types.h:ttc_gpio_speed_e)
 */
void ttc_gpio_init2_variable(ttc_Port_t* Port, ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed);

#define ttc_gpio_init_variable(Port, GPIOx_Pin, Type) ttc_gpio_init2_variable(Port, GPIOx_Pin, Type, tgs_Max);

//} Prototypes
/*{ Function Redefines

  Most functions in ttc_gpio are just redefinitions of low-level drivers.

  On most architectures, gpio operations require only some few assembly instructions.
  Adding another function call with argument passing can reduce maximum
  switching speed by a factor >4.

  You can see descriptions of equivalens functions below.
*/

/** initializes single port bit for use as input/ output with maximum speed by use of a state variable
 *
 * @param Port   points to structure to be filled with given data
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 * @param Type   mode to use (-> ttc_gpio_types.h:ttc_gpio_mode_e)
 */
// void ttc_gpio_init_variable(ttc_Port_t* Port, ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type);

/** allows to load gpio port data into a variable for use in later port operations
 *
 * @param Port   points to structure to be initialized by ttc_gpio_init2_variable()
 * @param GPIOx  one of GPIOA, GPIOB, GPIOC, GPIOD, GPIOE, GPIOF
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 * @return  filled out struct *Port
 *
 * Example:
 * Makefile
 *   COMPILE_OPTS += -DTTC_LED1=PIN_PA7
 *
 * Source
 *   ttc_Port_t Port;
 *   ttc_gpio_variable(&Port, TTC_LED1);
 *   ttc_gpio_init_variable(Port.GPIOx, Port.Pin, tgm_output_push_pull);
 */
// void ttc_gpio_variable(ttc_Port_t* Port, ttc_GPIO_BANK GPIOx, u8_t Pin);

/** initializes single port bit for use as input/ output with maximum speed
 *
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 * @param Type   mode to use (-> ttc_gpio_types.h:ttc_gpio_mode_e)
 */
void ttc_gpio_init(ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type);

/** initializes single port bit for use as input/ output with given speed
 *
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 * @param Type   mode to use (-> ttc_gpio_types.h:ttc_gpio_mode_e)
 */
void ttc_gpio_init2(ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed);

/** read current value of input pin by use of preinitialized port variable
 *
 * @param Port  filled out struct as being returned by ttc_gpio_variable()
 * @return      =1: voltage at input pin is logical 1; logical 0 otherwise
 */
// bool ttc_gpio_get_variable(ttc_Port_t* Port);

/** set output pin to logical one by use of preinitialized port variable
 *
 * @param Port  filled out struct as being returned by ttc_gpio_variable()
 */
// void ttc_gpio_set_variable(ttc_Port_t* Port);

/** set output pin to logical zero by use of preinitialized port variable
 *
 * @param Port  filled out struct as being returned by ttc_gpio_variable()
 */
// void ttc_gpio_clr_variable(ttc_Port_t* Port);

/** sets output port to given value
 *
 * @param Port  filled out struct as being returned by ttc_gpio_variable()
 * @param Value  =0: set output to logical 0; to logical 1 otherwise
 */
// void ttc_gpio_setn_variable(ttc_Port_t* Port, BOOL Value);

/** creates numeric 8-bit representing given GPIO-bank and pin number
 *
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 * @return       8-bit value representing given GPIO-pin in a memory friendly format
 */
// u8_t ttc_gpio_create_index8(ttc_GPIO_BANK GPIOx, u8_t Pin);

/** restores GPIO-bank and pin number from 8-bit value
 *
 * @param PhysicalIndex  8-bit value representing GPIO-bank + pin
 * @param GPIOx          loaded with GPIO bank (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 * @param Pin            loaded with pin number
 */
void ttc_gpio_from_index8(u8_t PhysicalIndex, ttc_GPIO_BANK* GPIOx, u8_t* Pin);

/** reads current value of single port bit (if initialized as input)
 *
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 * @param Type   mode to use (-> ttc_gpio_types.h:ttc_gpio_mode_e)
 * @return       =1: voltage at input pin is logical 1; logical 0 otherwise
 */
//bool ttc_gpio_get(GPIO_TypeDef* GPIOx, u8_t Pin);

/** sets output port to logical 1
 *
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 */
//void ttc_gpio_set(GPIO_TypeDef* GPIOx, u8_t Pin);

/** sets output port to logical 0
 *
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 */
//void ttc_gpio_clr(GPIO_TypeDef* GPIOx, u8_t Pin);

/** sets output port to given value
 *
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIOA, 7) = Pin_PA7
 * @param Pin    0..15
 * @param Type   mode to use (-> ttc_gpio_types.h:ttc_gpio_mode_e)
 * @param Value  =0: set output to logical 0; to logical 1 otherwise
 */
//void ttc_gpio_setn(GPIO_TypeDef* GPIOx, u8_t Pin, BOOL Value);

#ifdef TARGET_ARCHITECTURE_STM32
//X    #define ttc_gpio_init2(GPIOx_Pin, Type, Speed) stm32_gpio_init(GPIOx_Pin, stm32_map_Type(Type), stm32_map_Speed(Speed))
//X    #define ttc_gpio_init(GPIOx_Pin, Type)         stm32_gpio_init(GPIOx_Pin, stm32_map_Type(Type), stm32_map_Speed(tgs_Max))
    #define ttc_gpio_get_variable(Port)            stm32_gpio_get_variable(Port)
    #define ttc_gpio_set_variable(Port)            stm32_gpio_set_variable(Port)
    #define ttc_gpio_clr_variable(Port)            stm32_gpio_clr_variable(Port)
    #define ttc_gpio_variable(Port, GPIOx_Pin)     stm32_gpio_variable(Port, GPIOx_Pin)
    #define ttc_gpio_set(PORT)                     stm32_gpio_set(PORT)
    #define ttc_gpio_get(PORT)                     stm32_gpio_get(PORT)
    #define ttc_gpio_clr(PORT)                     stm32_gpio_clr(PORT)
    #define ttc_gpio_set2(BANK,PIN)                stm32_gpio_set(BANK,PIN)
    #define ttc_gpio_get2(BANK,PIN)                stm32_gpio_get(BANK,PIN)
    #define ttc_gpio_clr2(BANK,PIN)                stm32_gpio_clr(BANK,PIN)
    #define ttc_gpio_create_index8(GPIOx_Pin)      stm32_gpio_create_index8(GPIOx_Pin)
    #define ttc_gpio_from_index8(Index, GPIOx_Pin) stm32_gpio_from_index8(Index, GPIOx_Pin)
#endif

#ifdef TARGET_ARCHITECTURE_EM357
//X    #define ttc_gpio_init2(GPIOx_Pin, Type, Speed) em357_gpio_init2(GPIOx_Pin, em357_map_Type(Type), em357_map_Speed(Speed))
//X    #define ttc_gpio_init(GPIOx_Pin, Type)         em357_gpio_init2(GPIOx_Pin, em357_map_Type(Type), em357_map_Speed(tgs_Max))
    #define ttc_gpio_get(PORT)                     em357_portGet(PORT)
    #define ttc_gpio_set(PORT)                     em357_portSet(PORT)
    #define ttc_gpio_clr(PORT)                     em357_portClr(PORT)

    // yet undefined
    // #define ttc_gpio_create_index8(GPIOx_Pin)      em357_gpio_create_index8(GPIOx_Pin)
    // #define ttc_gpio_from_index8(Index, GPIOx_Pin) em357_gpio_from_index8(Index, GPIOx_Pin)
    // #define ttc_gpio_get_variable(Port)            em357_gpio_get_variable(Port)
    // #define ttc_gpio_set_variable(Port)            em357_gpio_set_variable(Port)
    // #define ttc_gpio_clr_variable(Port)            em357_gpio_clr_variable(Port)
    // #define ttc_gpio_variable(Port, GPIOx_Pin)     em357_gpio_variable(Port, GPIOx_Pin)
#endif

#define ttc_gpio_setn(PORT, VALUE)          if (VALUE) ttc_gpio_set2(PORT);         else ttc_gpio_clr2(PORT)
#define ttc_gpio_setn_variable(PORT, VALUE) if (VALUE) ttc_gpio_set_variable(PORT); else ttc_gpio_clr_variable(PORT)

//} Function Redefines
//{ check for missing low-level implementations

#ifndef ttc_gpio_get
  #warning Missing low level implementation for ttc_gpio_get()
#endif
#ifndef ttc_gpio_set
  #warning Missing low level implementation for ttc_gpio_set()
#endif
#ifndef ttc_gpio_clr
  #warning Missing low level implementation for ttc_gpio_clr()
#endif
#ifndef ttc_gpio_create_index8
  #warning Missing low level implementation for ttc_gpio_create_index8()
#endif
#ifndef ttc_gpio_from_index8
  #warning Missing low level implementation for ttc_gpio_from_index8()
#endif
#ifndef ttc_gpio_get_variable
  #warning Missing low level implementation for ttc_gpio_get_variable()
#endif
#ifndef ttc_gpio_set_variable
  #warning Missing low level implementation for ttc_gpio_set_variable()
#endif
#ifndef ttc_gpio_clr_variable
  #warning Missing low level implementation for ttc_gpio_clr_variable()
#endif
#ifndef ttc_gpio_variable
  #warning Missing low level implementation for ttc_gpio_variable()
#endif

//}
//{ shortcuts

#define gget   ttc_gpio_get
#define gset   ttc_gpio_set
#define gclr   ttc_gpio_clr
#define gget_v ttc_gpio_get_variable
#define gset_v ttc_gpio_set_variable
#define gclr_v ttc_gpio_clr_variable

//}
#endif
