/*{ ttc_memory.c ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Scheduler and architecture independent multimemorying support.
 * 
}*/
#include "ttc_memory.h"

#if TTC_MEMORY_ENUMERATE == 1
Base_t ttc_memory_CountAlloc = 0;
#endif

              void* ttc_memory_alloc(Base_t Size) {

    Base_t* Address = NULL;

#if TTC_MEMORY_ENUMERATE == 1
    Size += sizeof(Base_t);
#endif

#ifdef EXTENSION_300_scheduler_free_rtos
    Address = (Base_t*) pvPortMalloc(Size); // provided by heap_1.c, heap_2.c or heap_3.c
#endif

    Assert(Address != NULL, ec_OutOfMemory); // could not allocate memory!

#if TTC_MEMORY_ENUMERATE == 1
    *Address = ++ttc_memory_CountAlloc; // each memory block gets its own number at Address - sizeof(Base_t)
    Address++;
#endif

    return (void*) Address;
}
              void* ttc_memory_alloc_zeroed(Base_t Size) {

    void* Address = ttc_memory_alloc(Size);
    ttc_memory_set(Address, 0, Size);

    return Address;
}
               void ttc_memory_copy(void* Destination, const void* Source, Base_t AmountBytes) {
    Assert(Destination, ec_NULL);
    Assert(Source,      ec_NULL);

#if (TARGET_DATA_WIDTH == 32) // make use of 32-bit registers to speed up copying
    while (AmountBytes > 3) {
        *((u32_t*) Destination) = *((u32_t*) Source);
        Destination = (void*) (((u32_t) Destination) + 4);
        Source      = (void*) (((u32_t) Source)      + 4);
        AmountBytes -= 4;
    }
#else
#if (TARGET_DATA_WIDTH == 16) // make use of 16-bit registers to speed up copying
    while (AmountBytes > 1) {
        *((u16_t*) Destination) = *((u16_t*) Source);
        Destination = (void*) (((u32_t) Destination) + 2);
        Source      = (void*) (((u32_t) Source)      + 2);
        AmountBytes -= 2;
    }
#endif
#endif

    while (AmountBytes-- > 0) {
        *((u8_t*) Destination) = *((u8_t*) Source);
        Destination = (void*) (((u32_t) Destination) + 1);
        Source      = (void*) (((u32_t) Source)      + 1);
    }
}
               void ttc_memory_set(void* Destination, Base_t Fill, Base_t AmountBytes) {
    Assert(Destination, ec_NULL);

#if (TARGET_DATA_WIDTH == 32) // make use of 32-bit registers to speed up copying
    if (AmountBytes > 11) {
        while (AmountBytes > 3) {
            *((u32_t*) Destination) = Fill;
            Destination = (void*) (((u32_t) Destination) + 4);
            AmountBytes -= 4;
        }
    }
#else
#if (TARGET_DATA_WIDTH == 16) // make use of 16-bit registers to speed up copying

    if (AmountBytes > 5) {
        while (AmountBytes > 1) {
            *((u16_t*) Destination) = (u16_t) Fill);
            Destination = (void*) (((u32_t) Destination) + 2);
            AmountBytes -= 2;

        }
    }

#endif
#endif

    while (AmountBytes-- > 0) {
        *((u8_t*) Destination) = (u8_t) Fill;
        Destination = (void*) (((u32_t) Destination) + 1);
    }
}
             Base_t ttc_memory_compare(Base_t A, Base_t B, Base_t AmountBytes) {
    Base_t Index = 0;

#if (TARGET_DATA_WIDTH == 32) // make use of 32-bit registers to speed up copying
    if (AmountBytes > 11) {
        while (AmountBytes > 3) {
            if (*((u32_t*) A) != *((u32_t*) B) )  // words differ: will determine exact position later
                break;
            Index += 4;
            A += 4;
            B += 4;
            AmountBytes -= 4;
        }
    }
#endif
#if (TARGET_DATA_WIDTH == 16) // make use of 16-bit registers to speed up copying
    if (AmountBytes > 11) {
        while (AmountBytes > 3) {
            if (*((u32_t*) A) != *((u32_t*) B) ) // words differ: will determine exact position later
                break;
            Index += 2;
            A += 2;
            B += 2;
            AmountBytes -= 2;
        }
    }
#endif

    while (AmountBytes-- > 0) {
        if (*((u8_t*) A) != *((u8_t*) B) )
            return Index;
        A++;
        B++;
        Index++;
    }

    return 0;  // all checked bytes are the same
}
             void*  ttc_free(void* Block) {

#ifdef EXTENSION_300_scheduler_free_rtos
    vPortFree(Block); // provided by heap_1.c, heap_2.c or heap_3.c
    return NULL;
#endif

    return Block;
}
             Base_t ttc_memory_get_free_size() {

#ifdef EXTENSION_300_scheduler_free_rtos
    return (Base_t) xPortGetFreeHeapSize();
#endif

    return 0;
}
ttc_memory_block_t* ttc_memory_alloc_block(Base_t Size, u8_t Hint, void (*releaseBuffer)(ttc_memory_block_t* Block)) {
    ttc_memory_block_t* MemoryBlock = ttc_memory_alloc(sizeof(ttc_memory_block_t) + Size);

    MemoryBlock->Size          = 0;
    MemoryBlock->Hint          = Hint;
    MemoryBlock->releaseBuffer = releaseBuffer;
    MemoryBlock->Buffer        = ( (u8_t*) MemoryBlock ) + sizeof(ttc_memory_block_t);

    return MemoryBlock;
}
ttc_memory_block_t* ttc_memory_alloc_virtual_block(u8_t* Buffer, u8_t Hint, void (*releaseBuffer)(ttc_memory_block_t* Block)) {
    Assert(Buffer != NULL, ec_InvalidArgument);
    ttc_memory_block_t* MemoryBlock = ttc_memory_alloc(sizeof(ttc_memory_block_t));

    MemoryBlock->Size          = 0;
    MemoryBlock->Hint          = Hint;
    MemoryBlock->releaseBuffer = releaseBuffer;
    MemoryBlock->Buffer        = Buffer;

    return MemoryBlock;
}
               void ttc_memory_block_release(ttc_memory_block_t* Block) {
    Assert(Block, ec_InvalidArgument);
    Assert(Block->releaseBuffer, ec_UNKNOWN);  // no release function stored!

    Block->releaseBuffer(Block);
}
               BOOL ttc_memory_is_constant(const void* Address) {
    (void) Address; // avoids warning "unused variable"

#ifdef TARGET_ARCHITECTURE_STM32
    if (  ( ((u32_t) Address) > 0x08000000 ) && ( ((u32_t) Address) < 0x08040000 )  )
        return TRUE;
#endif
    return FALSE;
}
               BOOL ttc_memory_is_writable(void* Address) {
#ifdef TARGET_ARCHITECTURE_STM32
    if (  ( ((u32_t) Address) > 0x20000000) && ( ((u32_t) Address) < 0x20010000)  )
        return TRUE;
#endif
    return FALSE;
}
              void* _ttc_array_ref(Base_t* Size, Base_t Index) {
    Assert(Index < *Size, ec_ArrayIndexOutOfBound);
    return Size+1; // return pointer to first array element
}
             Base_t _ttc_array_index(Base_t* Size, Base_t Index) {
    Assert(Index < *Size, ec_ArrayIndexOutOfBound);
    return Index;
}
               u8_t ttc_memory_checkpointer(void* Pointer) {
    if (!Pointer)
        Assert_Halt_NULL();

    return 0;
}
               u8_t ttc_memory_checkpointer_within(void* Pointer, void* Min, void* Max) {
    Assert( (Pointer < Min) || (Pointer > Max), ec_IllegalPointer);
    return 0;
}
